package common.attributes;

import java.util.Map;

import common.collect.Maps;
import common.util.Displayable;
import common.util.ExtMath;
import common.util.Identifyable;

public enum Attribute implements Identifyable, Displayable {
	MAX_HEALTH("generic.maxHealth", "Maximale Gesundheit", 20.0D, 0.0D, 1024.0D, true),
	FOLLOW_RANGE("generic.followRange", "Kreaturen-Folgedistanz", 32.0D, 0.0D, 2048.0D, false),
	KNOCKBACK_RESISTANCE("generic.knockbackResistance", "Standfestigkeit", 0.0D, 0.0D, 1.0D, false),
	MOVEMENT_SPEED("generic.movementSpeed", "Geschwindigkeit", 0.699999988079071D, 0.0D, 1024.0D, true),
	ATTACK_DAMAGE("generic.attackDamage", "Angriffsschaden", 2.0D, 0.0D, 2048.0D, false),
	RADIATION("generic.radiation", "Strahlung", 0.0D, 0.0D, 16384.0D, false),
	RADIATION_RESISTANCE("generic.radiationResistance", "Strahlungsresistenz", 10.0D, 0.0D, 16384.0D, false),
	MANA_CAPACITY("generic.manaCapacity", "Mana-Kapazität", 0.0D, 0.0D, 2048.0D, false),
	MAGIC_RESISTANCE("generic.magicResistance", "Magieresistenz", 0.0D, 0.0D, 4096.0D, false),
	REINFORCEMENT_CHANCE("zombie.spawnReinforcements", "Zombie-Verstärkung", 0.0D, 0.0D, 1.0D, false),
	HORSE_JUMP_STRENGTH("horse.jumpStrength", "Pferdesprungstärke", 0.7D, 0.0D, 2.0D, true);

	private static final Map<String, Attribute> ATTRIBUTES = Maps.newHashMap();

	private final String name;
	private final String display;
	private final double defValue;
	private final double minValue;
	private final double maxValue;
	private final boolean watch;

	static {
		for(Attribute attr : values()) {
			ATTRIBUTES.put(attr.name, attr);
		}
	}

	public static Attribute getAttribute(String name) {
		return ATTRIBUTES.get(name);
	}

	private Attribute(String name, String display, double def, double min, double max, boolean watch) {
		if(name == null)
			throw new IllegalArgumentException("Name kann nicht null sein");
		else if(min > max)
			throw new IllegalArgumentException("Mindestwert kann nicht größer als Maximalwert sein");
		else if(def < min)
			throw new IllegalArgumentException("Standardwert kann nicht kleiner als Mindestwert sein");
		else if(def > max)
			throw new IllegalArgumentException("Standardwert kann nicht größer als Maximalwert sein");
		this.name = name;
		this.display = display;
		this.defValue = def;
		this.minValue = min;
		this.maxValue = max;
		this.watch = watch;
	}

	public String getName() {
		return this.name;
	}

	public String getDisplay() {
		return this.display;
	}

	public double getDefault() {
		return this.defValue;
	}

	public boolean isWatched() {
		return this.watch;
	}

	public double clamp(double value) {
		return ExtMath.clampd(value, this.minValue, this.maxValue);
	}
}
