package common.attributes;

import java.util.Collection;

import common.collect.Lists;
import common.log.Log;
import common.tags.TagObject;
import java.util.List;

public class Attributes {
	public static final AttributeModifier RUSHING_SPEED_MOD = new AttributeModifier(AttributeModifier.getModifierId("RushSpd"), "Attacking speed boost", 0.05, false, false);
	public static final AttributeModifier MAGE_POTSPEED_MOD = new AttributeModifier(AttributeModifier.getModifierId("MagePot"), "Drinking speed penalty", -0.25, false, false);
	public static final AttributeModifier ZOMBIE_BABY_MOD = new AttributeModifier(AttributeModifier.getModifierId("ZombSpd"), "Baby speed boost", 0.5, true);
	public static final AttributeModifier FLEEING_SPEED_MOD = new AttributeModifier(AttributeModifier.getModifierId("FleeSpd"), "Fleeing speed bonus", 2.0, true, false);
	public static final AttributeModifier SPRINT_SPEED_MOD = new AttributeModifier(AttributeModifier.getModifierId("Sprint"), "Sprinting speed boost", 0.3, true, false);
	public static final AttributeModifier MOUSE_SPEEDY_MOD = new AttributeModifier(AttributeModifier.getModifierId("SpeedyG"), "Mouse speed boost", 1.0, true);

	public static final long ITEM_VAL_ID = AttributeModifier.getModifierId("ItemVal");
	public static final long ITEM_DMG_ID = AttributeModifier.getModifierId("ItemDmg");

	public static List<TagObject> writeBaseAttributeMapToNBT(AttributeMap map) {
		List<TagObject> attrs = Lists.newArrayList();

		for(AttributeInstance instance : map.getAllAttributes()) {
			attrs.add(writeAttributeInstanceToNBT(instance));
		}

		return attrs;
	}

	private static TagObject writeAttributeInstanceToNBT(AttributeInstance instance) {
		TagObject tag = new TagObject();
		Attribute attr = instance.getAttribute();
		tag.setString("Name", attr.getName());
		tag.setDouble("Base", instance.getBaseValue());
		Collection<AttributeModifier> modifiers = instance.getModifiers();

		if(modifiers != null && !modifiers.isEmpty()) {
			List<TagObject> mods = Lists.newArrayList();

			for(AttributeModifier mod : modifiers) {
				if(mod.isSaved()) {
					mods.add(writeAttributeModifierToNBT(mod));
				}
			}

			tag.setList("Modifiers", mods);
		}

		return tag;
	}

	private static TagObject writeAttributeModifierToNBT(AttributeModifier mod) {
		TagObject tag = new TagObject();
		tag.setString("Name", mod.getName());
		tag.setDouble("Amount", mod.getAmount());
		tag.setBool("Multiply", mod.isMultiplied());
		tag.setLong("AttrId", mod.getID());
		return tag;
	}

	public static void setAttributeModifiers(AttributeMap map, List<TagObject> mods) {
		for(int i = 0; i < mods.size(); ++i) {
			TagObject mod = mods.get(i);
			AttributeInstance instance = map.getAttributeInstanceByName(mod.getString("Name"));

			if(instance != null) {
				applyModifiersToAttributeInstance(instance, mod);
			}
			else {
				Log.TICK.warn("Ignoriere unbekannte Attribute \'" + mod.getString("Name") + "\'");
			}
		}
	}

	private static void applyModifiersToAttributeInstance(AttributeInstance instance, TagObject tag) {
		instance.setBaseValue(tag.getDouble("Base"));

		if(tag.hasList("Modifiers")) {
			List<TagObject> mods = tag.getList("Modifiers");

			for(int i = 0; i < mods.size(); ++i) {
				AttributeModifier mod = readAttributeModifierFromNBT(mods.get(i));

				if(mod != null) {
					AttributeModifier old = instance.getModifier(mod.getID());

					if(old != null) {
						instance.removeModifier(old);
					}

					instance.applyModifier(mod);
				}
			}
		}
	}

	public static AttributeModifier readAttributeModifierFromNBT(TagObject tag) {
		long id = tag.getLong("AttrId");
		if(id == 0L)
			return null;

		try {
			return new AttributeModifier(id, tag.getString("Name"), tag.getDouble("Amount"), tag.getBool("Multiply"));
		}
		catch(Exception e) {
			Log.TICK.warn("Konnte Attribute nicht erstellen: " + e.getMessage());
			return null;
		}
	}
}
