package common.entity.item;

import common.entity.Entity;
import common.entity.EntityType;
import common.entity.types.EntityLiving;
import common.entity.types.IObjectData;
import common.model.ParticleType;
import common.tags.TagObject;
import common.world.World;

public class EntityTnt extends Entity implements IObjectData
{
    public int fuse;
    public int explosionSize;
    private EntityLiving tntPlacedBy;

    public EntityTnt(World worldIn)
    {
        super(worldIn);
        this.preventSpawning = true;
        this.setSize(0.98F, 0.98F);
    }

    public EntityTnt(World worldIn, double x, double y, double z, EntityLiving igniter, int power)
    {
        this(worldIn);
        this.setPosition(x, y, z);
        float f = (float)(Math.random() * Math.PI * 2.0D);
        this.motionX = (double)(-((float)Math.sin((double)f)) * 0.02F);
        this.motionY = 0.20000000298023224D;
        this.motionZ = (double)(-((float)Math.cos((double)f)) * 0.02F);
        this.fuse = 80;
        this.explosionSize = power & 7;
        this.prevX = x;
        this.prevY = y;
        this.prevZ = z;
        this.tntPlacedBy = igniter;
    }

    public EntityTnt(World worldIn, double x, double y, double z, int data)
    {
    	this(worldIn, x, y, z, null, data);
    }

    protected void entityInit()
    {
    }

    /**
     * returns if this entity triggers Block.onEntityWalking on the blocks they walk on. used for spiders and wolves to
     * prevent them from trampling crops
     */
    protected boolean canTriggerWalking()
    {
        return false;
    }

    /**
     * Returns true if other Entities should be prevented from moving through this Entity.
     */
    public boolean canBeCollidedWith()
    {
        return !this.dead;
    }

    /**
     * Called to update the entity's position/logic.
     */
    public void onUpdate()
    {
        this.prevX = this.posX;
        this.prevY = this.posY;
        this.prevZ = this.posZ;
        this.motionY -= 0.03999999910593033D;
        this.moveEntity(this.motionX, this.motionY, this.motionZ);
        this.motionX *= 0.9800000190734863D;
        this.motionY *= 0.9800000190734863D;
        this.motionZ *= 0.9800000190734863D;

        if (this.onGround)
        {
            this.motionX *= 0.699999988079071D;
            this.motionZ *= 0.699999988079071D;
            this.motionY *= -0.5D;
        }

        if (this.fuse-- <= 0)
        {
            this.setDead();

            if (!this.worldObj.client)
            {
                this.explode();
            }
        }
        else
        {
            this.handleWaterMovement();
            this.worldObj.spawnParticle(ParticleType.SMOKE_NORMAL, this.posX, this.posY + 0.5D, this.posZ, 0.0D, 0.0D, 0.0D);
        }
    }

    private void explode()
    {
        float f = 4.0F * (((float)this.explosionSize)+1.0f);
        this.worldObj.createExplosion(this, this.posX, this.posY + (double)(this.height / 16.0F), this.posZ, f, true);
    }

    /**
     * (abstract) Protected helper method to write subclass entity data to NBT.
     */
    protected void writeEntity(TagObject tagCompound)
    {
        tagCompound.setByte("Fuse", (byte)this.fuse);
        tagCompound.setByte("Power", (byte)this.explosionSize);
    }

    /**
     * (abstract) Protected helper method to read subclass entity data from NBT.
     */
    protected void readEntity(TagObject tagCompund)
    {
        this.fuse = tagCompund.getByte("Fuse");
        this.explosionSize = ((int)tagCompund.getByte("Power")) & 7;
    }

    /**
     * returns null or the entityliving it was placed or ignited by
     */
    public EntityLiving getTntPlacedBy()
    {
        return this.tntPlacedBy;
    }

    public float getEyeHeight()
    {
        return 0.0F;
    }
    
    public int getTrackingRange() {
    	return 160;
    }
    
    public int getUpdateFrequency() {
    	return 10;
    }
    
    public boolean isSendingVeloUpdates() {
    	return true;
    }
	
	public int getPacketData() {
		return this.explosionSize;
	}
	
//	public boolean hasSpawnVelocity() {
//		return false;
//	}
    
    public EntityType getType() {
    	return EntityType.EXPLOSIVE;
    }
}
