package common.packet;

import java.io.IOException;
import java.security.PrivateKey;
import java.security.PublicKey;

import common.network.IClientLoginHandler;
import common.network.Packet;
import common.network.PacketBuffer;
import common.util.EncryptUtil;

public class RPacketChallenge implements Packet<IClientLoginHandler> {
	private byte[] token;

	public RPacketChallenge() {
	}

	public RPacketChallenge(PublicKey pubkey, byte[] token) {
		this.token = EncryptUtil.encryptData(pubkey, token);
	}

	public final void readPacketData(PacketBuffer buf) throws IOException {
		this.token = buf.readByteArray();
	}

	public final void writePacketData(PacketBuffer buf) throws IOException {
		buf.writeByteArray(this.token);
	}

	public void processPacket(IClientLoginHandler handler) {
		handler.handleChallenge(this);
	}

	public byte[] getToken(PrivateKey key) {
		return EncryptUtil.decryptData(key, this.token);
	}
}
