package common.packet;

import java.io.IOException;

import common.entity.Entity;
import common.network.IClientPlayer;
import common.network.Packet;
import common.network.PacketBuffer;
import common.tags.TagObject;
import common.world.World;

public class SPacketUpdateEntityTags implements Packet<IClientPlayer>
{
    private int entityId;
    private TagObject tagCompound;

    public SPacketUpdateEntityTags()
    {
    }

    public SPacketUpdateEntityTags(int entityIdIn, TagObject tagCompoundIn)
    {
        this.entityId = entityIdIn;
        this.tagCompound = tagCompoundIn;
    }

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.entityId = buf.readVarInt();
        this.tagCompound = buf.readTag();
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeVarInt(this.entityId);
        buf.writeTag(this.tagCompound);
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(IClientPlayer handler)
    {
        handler.handleEntityTags(this);
    }

    public TagObject getTagCompound()
    {
        return this.tagCompound;
    }

    public Entity getEntity(World worldIn)
    {
        return worldIn.getEntityByID(this.entityId);
    }
}
