package server.command;

import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;

import common.collect.Lists;
import common.collect.Maps;

public record CachedExecutable(Executable executable, Map<String, Parameter> parameters, List<Parameter> positionals, Method method) {	
	public static CachedExecutable cacheExecutable(Executable executable) {
		Map<String, Parameter> parameters = Maps.newTreeMap();
		Map<String, Parameter> params = executable.getParameters();
		parameters.putAll(params);
		Parameter[] positions = new Parameter[parameters.size()];
		int pos = -1;
		for(Parameter param : params.values()) {
			if(param.positional()) {
				positions[param.position()] = param;
				pos = param.position() > pos ? param.position() : pos;
			}
			if(param.shorthand())
				parameters.put("" + param.shortName(), param);
		}
		List<Parameter> positionals = Lists.newArrayList();
		for(int z = 0; z <= pos; z++) {
			if(positions[z] == null)
				throw new NullPointerException("positions[" + z + "]");
			positionals.add(positions[z]);
		}
		List<Class<?>> classes = Lists.newArrayList(CommandEnvironment.class, Executor.class);
		for(Parameter param : executable.getParamList()) {
			ArgCombiner<?> combiner = param.combiner();
			if(combiner != null) {
				classes.add(combiner.getTypeClass());
				continue;
			}
			if(param.parsers().isEmpty()) {
				classes.add(boolean.class);
				continue;
			}
			for(ArgumentParser parser : param.parsers()) {
				classes.add(parser.getTypeClass(param.required()));
			}
		}
		Method method;
		try {
			method = executable.getClass().getDeclaredMethod("exec", classes.toArray(new Class<?>[classes.size()]));
		}
		catch(NoSuchMethodException e) {
			throw new RuntimeException(e);
		}
		return new CachedExecutable(executable, parameters, positionals, method);
	}
	
	public String name() {
		return this.executable.getName();
	}
	
	public String toString() {
		return this.executable.getName();
	}
}
