package server.command;

import java.util.Collection;

import common.dimension.Dimension;
import common.init.UniverseRegistry;
import common.util.Position;

public class DimensionParser extends CompletingParser {
	private final boolean useSender;
	
	public DimensionParser(String name, boolean useSender) {
		super(name);
		this.useSender = useSender;
	}

	public Object parse(CommandEnvironment env, String input) {
		Dimension dim = UniverseRegistry.getDimension(input);
		if(dim != null)
			return dim;
		int id = Integer.MIN_VALUE;
		try {
			id = Integer.parseInt(input);
		}
		catch(NumberFormatException e) {
		}
		dim = UniverseRegistry.getDimension(id);
		if(dim == null)
			throw new RunException("Unbekannte Dimension '%s'", input);
		return dim;
	}
	
	public Object getDefault(CommandEnvironment env) {
		if(!this.useSender)
			return null;
		Position pos = env.getExecutor().getExecPos();
		return pos == null ? null : UniverseRegistry.getDimension(pos.dim());
//		if(dim == null)
//			throw new ScriptException("Unbekannte Dimension '%s'");
	}
	
	public Collection<String> getCompletions(CommandEnvironment env) {
		return UniverseRegistry.getWorldNames();
	}

	public Class<?> getTypeClass(boolean required) {
		return Dimension.class;
	}
}
