package server.command.commands;

import java.util.List;
import java.util.Map.Entry;
import java.util.function.Function;

import common.collect.Lists;
import common.util.Util;
import server.command.ArgumentParser;
import server.command.CachedExecutable;
import server.command.Command;
import server.command.CommandEnvironment;
import server.command.Executor;
import server.command.Parameter;

public class CommandHelp extends Command {
	public CommandHelp(CommandEnvironment env) {
		super("help");

		this.setParamsOptional();
		this.addEnum("command", CachedExecutable.class, env.getExecutables().values());
	}
	
	public void exec(CommandEnvironment env, Executor exec, CachedExecutable command) {
		if(command == null) {
			for(CachedExecutable cmd : env.getExecutables().values()) {
				this.exec(env, exec, cmd);
			}
			return;
		}
		List<String> list = Lists.newArrayList();
		for(Entry<String, Parameter> entry : command.parameters().entrySet()) {
			Parameter param = entry.getValue();
			if(entry.getKey().length() == 1 && !param.positional() && param.shorthand()) {
				list.add("-" + param.shortName() + (param.parsers().isEmpty() ? " (" + param.name() + ")" : (param.parsers().size() == 1 && 
					param.parsers().get(0).getName().equals(param.name()) ? " <" + param.name() + ">" : 
					" (" + param.name() + ")" + Util.buildLines(" ", new Function<ArgumentParser, String>() {
					public String apply(ArgumentParser parser) {
						return "<" + parser.getName() + ">";
					}
				}, param.parsers()))));
			}
		}
		for(Parameter param : command.positionals()) {
			list.add((param.required() ? "<" : "[") + (param.parsers().size() == 1 && 
				param.parsers().get(0).getName().equals(param.name()) ? param.name() : 
				"(" + param.name() + ") " + Util.buildLines(" ", new Function<ArgumentParser, String>() {
				public String apply(ArgumentParser parser) {
					return "<" + parser.getName() + ">";
				}
			}, param.parsers())) + (param.required() ? ">" : "]"));
		}
		exec.logConsole("%s %s", command.name(), Util.buildLines(" ", list));
	}
}
