package server.command.commands;

import common.color.TextColor;
import common.init.Config;
import common.network.IPlayer;
import common.tags.TagObject;
import server.command.Command;
import server.command.CommandEnvironment;
import server.command.Executor;
import server.command.RunException;
import server.network.Player;
import server.util.Form;

public class CommandRegister extends Command {
	public CommandRegister() {
		super("register");

		this.addString("username", false, null, IPlayer.MAX_USER_LENGTH, IPlayer.VALID_USER);
		this.setParamsOptional();
		this.addString("password", false);
		this.addFlag("admin", 'a');
	}
	
	public void exec(CommandEnvironment env, Executor exec, String username, String password, boolean admin) {
		Player player = env.getServer().getPlayer(username);
		if(player != null)
			throw new RunException("Ein Spieler mit diesem Nutzernamen ist bereits online");
		TagObject tag = env.getServer().loadPlayerData(username);
		if(tag != null)
			throw new RunException("Ein Spieler mit diesem Nutzernamen ist bereits registriert");
		if(exec.isPlayer()) {
			if(password != null)
				throw new RunException("Bei Verwendung als Spieler darf kein Passwort angegeben werden");
			((Player)exec).displayForm(new Form() {
				private Field passwordField;
				private Field confirmField;
				
				protected void init() {
					this.passwordField = this.addPassword("Passwort", Config.minPassLength, IPlayer.MAX_PASS_LENGTH, "");
					this.confirmField = this.addPassword("Passwort bestätigen", Config.minPassLength, IPlayer.MAX_PASS_LENGTH, "");
				}
				
				public String getTitle() {
					return "Spieler " + username + " registrieren";
				}
				
				protected void accept() {
					if(!((Player)exec).isAdmin() || env.getServer().getPlayer(username) != null || env.getServer().loadPlayerData(username) != null) {
						exec.logConsole(TextColor.DRED + "Ein Fehler ist aufgetreten");
						return;
					}
					if(!this.passwordField.get().equals(this.confirmField.get())) {
						exec.logConsole(TextColor.RED + "Passwörter stimmen nicht überein");
						return;
					}
					TagObject user = new TagObject();
					user.setString("password", this.passwordField.get());
					env.getServer().writePlayerData(username, user);
					exec.logConsole(TextColor.GREEN + "Spieler %s registriert", username);
				}
			});
		}
		else if(exec.isConsole()) {
			if(password == null)
				throw new RunException("Bei Verwendung in der Konsole muss ein Passwort angegeben werden");
			TagObject user = new TagObject();
			user.setString("password", password);
			env.getServer().writePlayerData(username, user);
			exec.logConsole(TextColor.GREEN + "Spieler %s registriert", username);
		}
	}
}
