package server.command.commands;

import java.util.List;
import java.util.Set;

import common.collect.Lists;
import common.collect.Sets;
import common.entity.Entity;
import common.entity.types.EntityLiving;
import common.init.EntityRegistry;
import common.tags.TagObject;
import common.util.Util;
import common.util.Vec3;
import server.command.Command;
import server.command.CommandEnvironment;
import server.command.Executor;
import server.command.RunException;
import server.network.Player;
import server.world.WorldServer;

public class CommandSpawn extends Command {
	public CommandSpawn() {
		super("spawn");
		
		Set<String> names = Sets.newTreeSet();
		for(Class<? extends Entity> clazz : EntityRegistry.getAllClasses()) {
			names.add(EntityRegistry.getEntityString(clazz));
		}
		names.add("Lightning");
		this.addEnum("type", String.class, names);
		this.setParamsOptional();
		this.addVector("position", true, true);
		this.addWorld("dim", true);
		this.addTag("tag");
		
		this.addFlag("noinit", 'n');
		this.addInt("count", 'c', 1, 1024, 1);
		this.addTag("postTag", 'p');
	}
	
	public Object exec(CommandEnvironment env, Executor exec, String type, Vec3 pos, WorldServer world, TagObject tag, boolean noinit, int count, TagObject postTag) {
		if(type.equalsIgnoreCase("Lightning")) {
			for(int z = 0; z < count; z++) {
				int color = 0xffffff;
				if(tag != null && tag.hasInt("color"))
					color = tag.getInt("color");
				else if(tag != null && tag.hasString("color")) {
					try {
						color = Integer.parseUnsignedInt(tag.getString("color"), 16);
					}
					catch(NumberFormatException e) {
					}
				}
				world.strikeLightning(pos.xCoord, pos.yCoord, pos.zCoord, color, 
						tag != null && tag.hasInt("damage") ? tag.getInt("damage") : 0, tag != null && tag.hasBool("fire") && tag.getBool("fire"), 
						exec.isPlayer() && tag != null && tag.hasBool("summoned") && tag.getBool("summoned") ? ((Player)exec).getPresentEntity() : null);
			}
			exec.logConsole("%sBlitz bei %d, %d, %d in %s erschaffen", count == 1 ? "" : (count + "x "), (int)pos.xCoord, (int)pos.yCoord, (int)pos.zCoord, world.dimension.getFormattedName(false));
			return null;
		}
		else {
			List<String> spawned = Lists.newArrayList();
			for(int z = 0; z < count; z++) {
				Entity entity = EntityRegistry.createEntityByName(type, world);
				if(entity == null)
					throw new RunException("Objekt konnte nicht erzeugt werden");
				entity.setLocationAndAngles(pos.xCoord, pos.yCoord, pos.zCoord, world.rand.floatv() * 360.0f, 0.0f);
				if(tag != null) {
					TagObject ent = new TagObject();
					entity.writeTags(ent);
					ent.merge(tag);
					entity.readTags(ent);
				}
				if(!noinit && (entity instanceof EntityLiving))
					((EntityLiving)entity).onInitialSpawn(null);
				world.spawnEntityInWorld(entity);
				if(postTag != null) {
					TagObject ent = new TagObject();
					entity.writeTags(ent);
					ent.merge(postTag);
					entity.readTags(ent);
				}
				spawned.add("#" + entity.getId());
			}
			exec.logConsole("%s%s bei %d, %d, %d in %s erschaffen", count == 1 ? "" : (count + "x "), EntityRegistry.getEntityName(type), (int)pos.xCoord, (int)pos.yCoord, (int)pos.zCoord, world.dimension.getFormattedName(false));
			return Util.buildLines(",", spawned);
		}
	}
}
