package server.command.commands;

import common.dimension.Dimension;
import common.item.ItemSpaceNavigator;
import common.util.Position;
import server.command.Command;
import server.command.CommandEnvironment;
import server.command.Executor;
import server.command.RunException;
import server.world.WorldServer;

public class CommandTime extends Command {
	public CommandTime() {
		super("time");
		
		this.addString("time", false, "day", "night", "noon", "midnight", "sunrise", "sunset");
		
		this.addFlag("absolute", 'a');
	}

	private long parseInt(String input, long max) {
		long num;
		try {
			num = Long.parseLong(input);
		}
		catch(NumberFormatException e) {
			throw new RunException("'%s' ist keine gültige Zahl", input);
		}
		if(num < 0)
			throw new RunException("Die Zeit muss mindestens 0 betragen", num);
		else if(num > max)
			throw new RunException("Die Zeit darf höchstens %d betragen", num, max);
		return num;
	}

	private long parseDayTime(Dimension dim, String arg) {
		if(arg.equalsIgnoreCase("sunrise"))
			return dim.getRotationalPeriod() / 6L;
		else if(arg.equalsIgnoreCase("day"))
			return dim.getRotationalPeriod() * 5L / 24L;
		else if(arg.equalsIgnoreCase("noon"))
			return dim.getRotationalPeriod() / 2L;
		else if(arg.equalsIgnoreCase("sunset"))
			return dim.getRotationalPeriod() * 5L / 6L;
		else if(arg.equalsIgnoreCase("night"))
			return dim.getRotationalPeriod() * 21L / 24L;
		else if(arg.equalsIgnoreCase("midnight"))
			return 0L;
		else if(arg.startsWith("@"))
			return this.parseInt(arg.substring(1), dim.getRotationalPeriod() - 1);
		else
			return -1L;
	}
	
	private long parseTime(Dimension dim, String arg) {
		long t;
		if(arg.toLowerCase().endsWith("y"))
			t = dim.getOrbitalPeriod();
		else if(arg.toLowerCase().endsWith("d"))
			t = dim.getRotationalPeriod();
		else if(arg.toLowerCase().endsWith("h"))
			t = 1000L;
		else
			t = -1L;
		arg = t == -1L ? arg : arg.substring(0, arg.length() - 1);
		t = t < 1L ? 1L : t;
		return t * this.parseInt(arg, Long.MAX_VALUE);
	}
	
	public Object exec(CommandEnvironment env, Executor exec, String timeStr, boolean absolute) {
		Position pos = exec.getExecPos();
		WorldServer world = pos == null ? env.getServer().getSpace() : env.getServer().getWorld(pos.dim());
		long time = absolute ? 0L : world.getDayTime();
		long fwd = parseDayTime(world.dimension, timeStr);
		if(fwd >= 0L) {
			if(absolute) {
				time += fwd;
			}
			else {
				time -= world.getDayTime() % world.dimension.getRotationalPeriod();
				time += fwd;
				time += time <= world.getDayTime() ? world.dimension.getRotationalPeriod() : 0L;
			}
		}
		else {
			time += parseTime(world.dimension, timeStr);
		}
		for(WorldServer dim : env.getServer().getWorlds()) {
			dim.setDayTime(time);
			dim.resetWeather();
		}
		exec.logConsole("Zeit auf %s gesetzt", ItemSpaceNavigator.formatImperialTime(world, false));
		return time;
	}
}
