package server.network;

import common.network.IHandshakeHandler;
import common.network.NetConnection;
import common.network.NetHandler;
import common.network.PacketRegistry;
import common.packet.HPacketHandshake;
import common.packet.RPacketDisconnect;
import common.util.Util;
import server.Server;

public class HandshakeHandler extends NetHandler implements IHandshakeHandler
{
	private final Server server;
    private final NetConnection networkManager;

    public HandshakeHandler(Server serverIn, NetConnection netManager)
    {
        this.server = serverIn;
        this.networkManager = netManager;
    }
    
    public void onDisconnect(String reason)
    {
    }
    
    public void processHandshake(HPacketHandshake packetIn)
    {
    	if(packetIn.getProtocolVersion() == 0) {
    		this.networkManager.closeChannel("Inkompatibel!");
    		return;
    	}
        this.networkManager.setConnectionState(PacketRegistry.LOGIN);
        if (packetIn.getProtocolVersion() != Util.PROTOCOL)
        {
        	String comp;
        	if(packetIn.getProtocolVersion() < Util.PROTOCOL)
        		comp = "Der Server nutzt eine neuere Version: " + Util.VERSION;
        	else
        		comp = "Der Server nutzt eine ältere Version: " + Util.VERSION;
            this.networkManager.sendPacket(new RPacketDisconnect(comp));
            this.networkManager.closeChannel(comp);
        }
        else
        {
        	LoginHandler handler = new LoginHandler(this.server, this.networkManager);
            this.networkManager.setNetHandler(handler);
            handler.sendLoginPacket();
        }
	}
}
