package server.worldgen.structure;

import java.util.List;
import java.util.Set;

import common.biome.Biome;
import common.collect.Sets;
import common.rng.Random;
import common.tags.TagObject;
import server.world.WorldServer;

public class MapGenVillage extends MapGenStructure
{
    public static final Set<Biome> villageSpawnBiomes = Sets.<Biome>newHashSet(Biome.PLAINS, Biome.DESERT, Biome.SAVANNA);

    /** World terrain type, 0 for normal, 1 for flat map */
    private int terrainType;
    private int distance;
    private int field_82666_h;

    public MapGenVillage()
    {
        this.distance = 32;
        this.field_82666_h = 8;
    }

//    public MapGenVillage(Map<String, String> p_i2093_1_)
//    {
//        this();
//
//        for (Entry<String, String> entry : p_i2093_1_.entrySet())
//        {
//            if (((String)entry.getKey()).equals("size"))
//            {
//                this.terrainType = MathHelper.parseIntWithDefaultAndMax((String)entry.getValue(), this.terrainType, 0);
//            }
//            else if (((String)entry.getKey()).equals("distance"))
//            {
//                this.distance = MathHelper.parseIntWithDefaultAndMax((String)entry.getValue(), this.distance, this.field_82666_h + 1);
//            }
//        }
//    }

    public String getStructureName()
    {
        return "Village";
    }

    protected boolean canSpawnStructureAtCoords(int chunkX, int chunkZ)
    {
        int i = chunkX;
        int j = chunkZ;

        if (chunkX < 0)
        {
            chunkX -= this.distance - 1;
        }

        if (chunkZ < 0)
        {
            chunkZ -= this.distance - 1;
        }

        int k = chunkX / this.distance;
        int l = chunkZ / this.distance;
        Random random = this.worldObj.getRng(k, l, 10387312);
        k = k * this.distance;
        l = l * this.distance;
        k = k + random.zrange(this.distance - this.field_82666_h);
        l = l + random.zrange(this.distance - this.field_82666_h);

        if (i == k && j == l)
        {
            boolean flag = this.worldObj.getBiomeGenerator().areBiomesViable(i * 16 + 8, j * 16 + 8, 0, villageSpawnBiomes);

            if (flag)
            {
                return true;
            }
        }

        return false;
    }

    protected StructureStart getStructureStart(int chunkX, int chunkZ)
    {
        return new MapGenVillage.Start(this.worldObj, this.rand, chunkX, chunkZ, this.terrainType);
    }

    public static class Start extends StructureStart
    {
        private boolean hasMoreThanTwoComponents;

        public Start()
        {
        }

        public Start(WorldServer worldIn, Random rand, int x, int z, int size)
        {
            super(x, z);
            List<StructureVillage.PieceWeight> list = StructureVillage.getStructureVillageWeightedPieceList(rand, size);
            StructureVillage.Start structurevillagepieces$start = new StructureVillage.Start(worldIn.getBiomeGenerator(), 0, rand, (x << 4) + 2, (z << 4) + 2, list, size);
            this.components.add(structurevillagepieces$start);
            structurevillagepieces$start.buildComponent(structurevillagepieces$start, this.components, rand);
            List<StructureComponent> list1 = structurevillagepieces$start.field_74930_j;
            List<StructureComponent> list2 = structurevillagepieces$start.field_74932_i;

            while (!list1.isEmpty() || !list2.isEmpty())
            {
                if (list1.isEmpty())
                {
                    int i = rand.zrange(list2.size());
                    StructureComponent structurecomponent = (StructureComponent)list2.remove(i);
                    structurecomponent.buildComponent(structurevillagepieces$start, this.components, rand);
                }
                else
                {
                    int j = rand.zrange(list1.size());
                    StructureComponent structurecomponent2 = (StructureComponent)list1.remove(j);
                    structurecomponent2.buildComponent(structurevillagepieces$start, this.components, rand);
                }
            }

            this.updateBoundingBox();
            int k = 0;

            for (StructureComponent structurecomponent1 : this.components)
            {
                if (!(structurecomponent1 instanceof StructureVillage.Road))
                {
                    ++k;
                }
            }

            this.hasMoreThanTwoComponents = k > 2;
        }

        public boolean isSizeableStructure()
        {
            return this.hasMoreThanTwoComponents;
        }

        public void writeTags(TagObject tagCompound)
        {
            super.writeTags(tagCompound);
            tagCompound.setBool("Valid", this.hasMoreThanTwoComponents);
        }

        public void readTags(TagObject tagCompound)
        {
            super.readTags(tagCompound);
            this.hasMoreThanTwoComponents = tagCompound.getBool("Valid");
        }
    }
}
